@@ -4,7 +4,7 @@
  * Gnome Chemistry Utils
  * gcu/application.cc 
  *
- * Copyright (C) 2005-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2005-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -24,6 +24,8 @@
 
 #include "config.h"
 #include "application.h"
+#include <libgnomevfs/gnome-vfs-ops.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include <glade/glade.h>
 #include <gconf/gconf-client.h>
 #include <gtk/gtklabel.h>
@@ -39,13 +41,14 @@ Application::Application (string name, string datadir, char const *help_name, ch
 	Name = name;
 	char const *szlang = getenv ("LANG");
 	string lang = (szlang)? szlang: "C";
-	string HelpName = help_name? help_name: Name;
-	HelpFilename = datadir + string ("/gnome/help/") + HelpName + string ("/") + lang + string ("/") + HelpName + ".xml";
-	struct stat buf;
-	gint err;
-	err = stat (HelpFilename.c_str (), &buf);
-	if (err)
-		HelpFilename = datadir + string ("/gnome/help/") + HelpName + string ("/C/") + HelpName + ".xml";
+	HelpName = help_name? help_name: Name;
+	HelpFilename = string ("file://") + datadir + string ("/gnome/help/") + HelpName + string ("/") + lang + string ("/") + HelpName + ".xml";
+	GnomeVFSURI *uri = gnome_vfs_uri_new (HelpFilename.c_str ());
+	bool exists = gnome_vfs_uri_exists (uri);
+	gnome_vfs_uri_unref (uri);
+	if (!exists) {
+		HelpFilename = string ("file://") + datadir + string ("/gnome/help/") + HelpName + string ("/C/") + HelpName + ".xml";
+	}
 	GConfClient* cli = gconf_client_get_default ();
 	if (cli) {
 		const char *value;
@@ -76,6 +79,7 @@ Application::Application (string name, string datadir, char const *help_name, ch
 	GdkScreen *screen = gdk_screen_get_default ();
 	m_ScreenResolution = (unsigned) rint (gdk_screen_get_width (screen) * 25.4 / gdk_screen_get_width_mm (screen));
 	m_ImageResolution = m_ScreenResolution;
+	m_ImageHeight = m_ImageWidth = 300;
 	m_RecentManager = gtk_recent_manager_new ();
 
 	// check supported pixbuf formats
@@ -110,7 +114,7 @@ void Application::OnHelp (string tag)
 	argv[0] = (char*) HelpBrowser.c_str();
 	string path = HelpFilename;
 	if (tag.length ())
-		path += string("#") + Name + string ("-") + tag;
+		path += string("#") + HelpName + string ("-") + tag;
 	argv[1] = (char*) path.c_str ();
 	g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
 		NULL, NULL, NULL, NULL);
@@ -120,10 +124,10 @@ bool Application::HasHelp ()
 {
 	if (!HelpBrowser.length () || !HelpFilename.length ())
 		return false;
-	struct stat buf;
-	gint err;
-	err = stat (HelpFilename.c_str (), &buf);
-	return (err)? false: true;	
+	GnomeVFSURI *uri = gnome_vfs_uri_new (HelpFilename.c_str ());
+	bool err = gnome_vfs_uri_exists (uri);
+	gnome_vfs_uri_unref (uri);
+	return err;	
 }
 
 void Application::SetCurDir (char const* dir)
@@ -159,6 +163,16 @@ static void on_res_changed (GtkSpinButton *btn, Application *app)
 	app->SetImageResolution (gtk_spin_button_get_value_as_int (btn));
 }
 
+static void on_width_changed (GtkSpinButton *btn, Application *app)
+{
+	app->SetImageWidth (gtk_spin_button_get_value_as_int (btn));
+}
+
+static void on_height_changed (GtkSpinButton *btn, Application *app)
+{
+	app->SetImageHeight (gtk_spin_button_get_value_as_int (btn));
+}
+
 GtkWidget *Application::GetImageResolutionWidget ()
 {
 	GladeXML *xml = glade_xml_new (GLADEDIR"/image-resolution.glade", "res-table", NULL);
@@ -170,6 +184,20 @@ GtkWidget *Application::GetImageResolutionWidget ()
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (w), m_ImageResolution);
 	g_signal_connect (G_OBJECT (w), "value-changed", G_CALLBACK (on_res_changed), this);
 	w = glade_xml_get_widget (xml, "res-table");
+	g_object_unref (G_OBJECT (xml));
+	return w;
+}
+
+GtkWidget *Application::GetImageSizeWidget ()
+{
+	GladeXML *xml = glade_xml_new (GLADEDIR"/image-size.glade", "size-table", NULL);
+	GtkWidget *w = glade_xml_get_widget (xml, "width");
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (w), m_ImageWidth);
+	g_signal_connect (G_OBJECT (w), "value-changed", G_CALLBACK (on_width_changed), this);
+	w = glade_xml_get_widget (xml, "height");
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (w), m_ImageHeight);
+	g_signal_connect (G_OBJECT (w), "value-changed", G_CALLBACK (on_height_changed), this);
+	w = glade_xml_get_widget (xml, "size-table");
 	return w;
 }
 
@@ -197,3 +225,4 @@ char const *Application::GetPixbufTypeName (string& filename, char const *mime_t
 	g_strfreev (exts);
 	return gdk_pixbuf_format_get_name (format);
 }
+
